#-*- coding: utf-8 -*-
"""Database module to store various search engine data."""
import sys
import bsddb
import Pyro.util
import logging
import threading

from time import time

class Database(object):
    """
    Database storing various search engine data including:
      - Statistics
        - Crawler statistics
        - Indexing statistics
      - Outlink data
    
    """
    def __init__(self, db_path):
        """
        Initialize the Database object.

        @type db_path: string
        @param db_path: Path to the database directory.
        """
        self.logger = Pyro.util.UserLogger()
        env = bsddb.db.DBEnv()
        env.set_tx_max(160)
	env.set_flags(bsddb.db.DB_TXN_NOSYNC, 1)
        env.open(db_path, bsddb.db.DB_CREATE
                         |bsddb.db.DB_INIT_LOCK
                         |bsddb.db.DB_THREAD
                         |bsddb.db.DB_INIT_MPOOL
                         |bsddb.db.DB_INIT_LOG
                         |bsddb.db.DB_INIT_TXN
                         |bsddb.db.DB_RECOVER
                         )

        self.env = env

        self.database = bsddb.db.DB(env)

        self.database.open('database.db', 'database', dbtype=bsddb.db.DB_BTREE,
                        flags=bsddb.db.DB_CREATE
                              |bsddb.db.DB_THREAD
                              |bsddb.db.DB_AUTO_COMMIT
                              )

    def __del__(self):
        self.close()

    def close(self):
        self.logger.msg("Database", "Database is closing...")
        self.database.close()

    def increment(self, namespace, key):
        """
        Increases a value in key.

        @type  namespace: string
        @param namespace: Namespace of the key.
        @type  key: string
        @param key: Key.
        """
        txn = self.env.txn_begin()
        try:
            total = self.get(namespace, key)
            if not total: total = 0
            self.put(namespace, key, int(total)+1)
        except:
            txn.abort()
        else:
            txn.commit()

    def decrement(self, namespace, key):
        """
        Decreases a value in key.

        @type  namespace: string
        @param namespace: Namespace of the key.
        @type  key: string
        @param key: Key.
        """
        txn = self.env.txn_begin()
        try:
            total = self.get(namespace, key)
            if not total: total = 0
            self.put(namespace, key, int(total)-1)
        except:
            txn.abort()
        else:
            txn.commit()
    
    def get(self, namespace, key):
        """
        Get the value associated with the key.

        @type  namespace: string
        @param namespace: Namespace of the key.
        @type  key: string
        @param key: Key.
        @rtype: string
        @return: The value associated with the key.
        """
        return self.database.get(namespace + "." + str(key))

    def items(self, namespace):
        """
        Get the key, value pair items associated with the namespace.

        @type  namespace: string
        @param namespace: Namespace of the key.
        @rtype: list
        @return: The list of key, value pair associated with the namespace.
        """
        prefix = namespace + "."
        rows = self.get_by_prefix(prefix)
        result = []
        for row in rows:
            key, value = row
            key = key.split(".", 1)[1]
            result.append((key, value))
        return result

    def get_all(self, namespace, key):
        """
        Get all the previous and present key, value pair items associated with
        the key. This returns all the previously set values too.

        >>> database.get_all("namespace", "key")
        [("namespace:key:110220340403", "asd"),
        ("namespace:key:110220340501", "new value"),
        ("namespace:key:110220362349", "yet a more new value")]

        @type  namespace: string
        @param namespace: Namespace of the key.
        @type  key: string
        @param key: Key.
        @rtype: list
        @return: The list of key, value pair associated with the key.
        """
        start = namespace + ":" + key + ":"
        return self.get_by_prefix(start)

    def get_index_range(self, namespace, key, start, end):
        if not isinstance(start, int): start = int(start)
        if not isinstance(end, int): end = int(end)
        if isinstance(start, int) or isinstance(start, long):
            start = "%20d" % start
        if isinstance(end, int) or isinstance(end, long):
            end = "%20d" % end
        return self.get_range(namespace + ":" + key + ":" + start,
                                 namespace + ":" + key + ":" + end)

    def get_range(self, start, end):
        start = str(start)
        end = str(end)
        cursor = self.database.cursor()
        data = []
        data.append(cursor.set_range(start))
        while True:
            val = cursor.next()
            if not val:
                break
            if val[0] <= end:
                data.append(val)
            else:
                break
        return data

    def get_by_prefix(self, prefix):
        """
        Get all the internal_key, value pairs with prefix matching of the prefix argument.

        >>> database.get_by_prefix("namespace:key")
        [("namespace:key:110220340403", "asd"),
        ("namespace:key:110220340501", "new value"),
        ("namespace:key:110220362349", "yet a more new value")]

        @type  prefix: string
        @param prefix: Prefix of the internal key.
        @rtype: list
        @return: The list of key, value pair associated with the prefix internal key.
        """
        cursor = self.database.cursor()
        data = []
        data.append(cursor.set_range(prefix))
        while True:
            tmp = cursor.next()
            if not tmp: break
            if tmp[0].startswith(prefix):
                data.append(tmp)
            else:
                break
        return data

    def put(self, namespace, key, data, index=None):
        """
        Put the value associated with the key with index timestamp when
        index is not given.

        @type  namespace: string
        @param namespace: Namespace of the key.
        @type  key: string
        @param key: Key.
        @type  data: string
        @param data: Data to store.
        @type  index: string
        @param index: Custom index value to use insead of unix timestamp.
        """
        if index == None:
            index = int(time()*1000)
        assert index % 10 <= 20
        self.database.put(namespace + "." + str(key), str(data))
        self.database.put(namespace + ":" + str(key) + ":" + "%20d"%(index), str(data))
