#-*- coding: utf-8 -*-
"""Storage module to store crawled page data."""

import sys
import bsddb
import threading

import Pyro.util

class Storage(object):
    """
    Web page storage.

    Example:
    >>> st = Storage("/tmp/db")
    >>> st.add("http://ara.kaist.ac.kr/", "<html>...</html>")
    >>> st.get_data("http://ara.kaist.ac.kr/")
    "<html>...</html>"
    """
    def __init__(self, db_path):
        """
        Creates the storage database.

        @type db_path: string
        @param db_path: Path to the database directory.
        """
        self.logger = Pyro.util.UserLogger()
        env = bsddb.db.DBEnv()
        env.open(db_path, bsddb.db.DB_CREATE
                         |bsddb.db.DB_INIT_LOCK
                         |bsddb.db.DB_THREAD
                         |bsddb.db.DB_INIT_MPOOL
                         |bsddb.db.DB_INIT_LOG
                         |bsddb.db.DB_INIT_TXN
                         |bsddb.db.DB_RECOVER
                         )

        self.data_db = bsddb.db.DB(env)
        self.url_db = bsddb.db.DB(env)
        self.type_db = bsddb.db.DB(env)
        self.dup_db = bsddb.db.DB(env)
        self.pagerank_db = bsddb.db.DB(env)

        self.data_db.open('storage.db', 'storage', dbtype=bsddb.db.DB_BTREE, 
                     flags=bsddb.db.DB_CREATE
                            |bsddb.db.DB_THREAD
                            |bsddb.db.DB_AUTO_COMMIT
                            )

        self.url_db.open('storage.db', 'url', dbtype=bsddb.db.DB_BTREE, 
                     flags=bsddb.db.DB_CREATE
                            |bsddb.db.DB_THREAD
                            |bsddb.db.DB_AUTO_COMMIT
                            )
        self.type_db.open('storage.db', 'type', dbtype=bsddb.db.DB_BTREE, 
                     flags=bsddb.db.DB_CREATE
                            |bsddb.db.DB_THREAD
                            |bsddb.db.DB_AUTO_COMMIT
                            )

        self.dup_db.open('duplicate.db', 'duplicate', dbtype=bsddb.db.DB_BTREE, 
                     flags=bsddb.db.DB_CREATE
                            |bsddb.db.DB_THREAD
                            |bsddb.db.DB_AUTO_COMMIT
                            )

        self.pagerank_db.open('pagerank.db', 'pagerank', dbtype=bsddb.db.DB_BTREE, 
                     flags=bsddb.db.DB_CREATE
                            |bsddb.db.DB_THREAD
                            |bsddb.db.DB_AUTO_COMMIT
                            )

        self.lock = threading.RLock()

        cursor = self.url_db.cursor()
        r = cursor.last()

        if not r:
            self.last_id = 0
        else:
            self.last_id = int(r[0])
        cursor.close()

    def __del__(self):
        self.close()

    def close(self):
        self.logger.msg("Storage", "Storage is closing...")
        self.url_db.close()
        self.data_db.close()
        self.dup_db.close()

    def add(self, url, data, type):
        """
        Saves the new page to the database.

        @type  url: string
        @param url: URL of the page.
        @type  data: string
        @param data: Binary string for the content of the page.
        @type  type: string
        @param data: type of the content
        """
        self.last_id += 1
        self.url_db.put("%020d"%(self.last_id), str(url))
        self.data_db.put("%020d"%(self.last_id), str(data))
        self.type_db.put("%020d"%(self.last_id), str(type))
        self.dup_db.put(str(url), "%020d"%(self.last_id))
        return self.last_id

    def get_last_id(self):
        """
        Returns the last id of the database.

        @rtype: number
        @return: Last if of the database.
        """
        return self.last_id

    def modify(self, id, data):
        """
        Modify the content of the page.

        @type  id: number
        @param id: doc_id of the page.
        @type  data: string
        @param data: Binary string for the content of the page.
        """
        self.data_db.put("%020d"%id, str(data))

    def delete(self, id):
        """
        Delete the page.

        @type  id: number
        @param id: doc_id of the page.
        """
        self.url_db.delete("%020d"%id)
        self.data_db.delete("%020d"%id)
        self.type_db.delete("%020d"%id)

    def get_url(self, id):
        """
        Get the url of the page with doc_id.

        @type  id: number
        @param id: doc_id of the page.
        @rtype: string
        @return: url of the page.
        """
        return self.url_db.get("%020d"%id)

    def get_type(self, id):
        """
        Get the type of the page with doc_id.

        @type  id: number
        @param id: doc_id of the page.
        @rtype: string
        @return: type of the page.
        """
        return self.type_db.get("%020d"%id)

    def get_data(self, id):
        """
        Get the content of the page with doc_id.

        @type  id: number
        @param id: doc_id of the page.
        @rtype: string
        @return: Content string of the page.
        """
        return self.data_db.get("%020d"%id)

    def get_id(self, url):
        """
        Get the doc_id of the page with url.

        @type  url: string
        @param url: URL of the page.
        @rtype: number
        @return: doc_id of the page.
        """
        ret = self.dup_db.get(str(url))
        if ret == None: return None
        return int(ret)

    def get_pagerank(self, id):
	return self.pagerank_db.get("%020d" % id)

    def set_pagerank(self, docid_rank):
	for docid, rank in docid_rank:
	    self.pagerank_db.put("%020d" % docid, str(rank))
