#-*- coding: utf-8 -*-
import re
import imp
import traceback
import threading
import os
import time
import string
import logging

import Pyro.core
import Pyro.util

from ksearch.frontend.template import loader
from ksearch.frontend.template.context import Context
from ksearch.util import http_date

SERVER_NAME = 'SPARCS Web Server'
SERVER_VERSION = '0.1'

class Template(string.Template): # To be deprecated
    def __init__(self, str=None, filepath=None):
        if str:
            string.Template.__init__(self, str)
        elif filepath:
            fp = os.path.join(os.path.dirname(__file__), filepath)
            f = open(fp)
            buf = f.read().decode('utf8')
            f.close()
            string.Template.__init__(self, buf)
        else:
            raise InvalidArgsException()

class Response(object):
    """HTTP 응답 클래스. 기본값으로 200 OK를 나타낸다."""

    DEFAULT_CONTENT_TYPE = 'text/html'
    DEFAULT_CHARSET = 'utf-8'

    def __init__(self, response_code='200 OK', headers=None, body=''):
        self.response_code = response_code
        self._headers = {}
        if headers is None:
            self['Content-Type'] = '%s; charset=%s' % (self.DEFAULT_CONTENT_TYPE, self.DEFAULT_CHARSET)
        else:
            for h, v in headers.items():
                self[h] = v
        self.body = body

    def __getitem__(self, header):
        return self._headers[header.lower()]

    def __setitem__(self, header, value):
        self._headers[header.lower()] = value

    def __delitem__(self, header):
        try:
            del self._headers[header.lower()]
        except KeyError:
            pass

    def has_header(self, header):
        return self._headers.has_key(header.lower())
    __contains__ = has_header

    def get_headers(self):
        """헤더의 iterator를 반환한다."""
        for h, v in self._headers.items():
            if type(v) is not string:
                self._headers[h] = str(v)
        return self._headers.items()

class ResponseNotFound(Response):
    """지정한 주소의 내용을 찾을 수 없음을 나타내는 HTTP 응답 클래스."""

    def __init__(self, url='', server_info=None):
        Response.__init__(self, '404 Not Found',
                          {'Content-Type': 'text/html; charset=%s' % Response.DEFAULT_CHARSET})
        if server_info is None:
            server_info = {'server_name': SERVER_NAME, 'server_version': SERVER_VERSION}
        t = loader.get_template('404.html')
        c = Context({'url': url, 'server_info': server_info})
        self.body = t.render(c)

class ResponseRedirect(Response):
    """다른 주소로 redirect 시키기 위한 HTTP 응답 클래스."""

    def __init__(self, redirect_url):
        Response.__init__(self, '302 Found', {'Location': redirect_url})

class ResponseNotModified(Response):
    """브라우저가 캐시를 지원할 때, 이 주소에서 이전에 가져간 내용을 그대로 사용할 수 있음을 나타내는 HTTP 응답 클래스"""

    def __init__(self, date=0):
        Response.__init__(self, '304 Not Modified', None, '')
        if date == 0:
            date = time.time()
        if type(date) == float or type(date) == int:
            date = http_date(date)
        self['Date'] = date

class ResponseError(Response):
    """서버 내부에서 에러가 발생했음을 알리는 HTTP 응답 클래스"""

    def __init__(self, msg='', server_info=None):
        Response.__init__(self, '500 Internal Error',
                          {'Content-Type': 'text/html; charset=%s' % Response.DEFAULT_CHARSET})
        if server_info is None:
            server_info = {'server_name': SERVER_NAME, 'server_version': SERVER_VERSION}
        t = loader.get_template('500.html')
        c = Context({'msg': msg, 'server_info': server_info})
        self.body = t.render(c)


class SimpleFramework(object):
    """SimpleFramework 본체. Callable 인터페이스를 가지며 WSGI 서버 쪽에서 바로 호출된다."""

    def __init__(self, urls, debug=True, namespace="test_world"):
        self.logger = logging.getLogger('frontend')
        self.server_info = {'server_name': SERVER_NAME, 
                            'server_version': SERVER_VERSION}
        self.url_mapping = []
        self.module_desc = {}
        self.debug = debug
        self.namespace = namespace
        for u in urls:
            regex = re.compile(u[0])
            module_path = '.'.join(u[1].split('.')[:-1])
            function_name = u[1].split('.')[-1]
            file, pathname, description = imp.find_module(module_path, ['frontend'])
            module = imp.load_module(module_path, file, pathname, description)
            module.object_namespace = self.namespace
            file.close()
            self.url_mapping.append((regex, module, module_path, function_name))

    def renew_modules(self):
        for i in range(len(self.url_mapping)):
            m = self.url_mapping[i]
            m_path = m[2]
            file, pathname, description = imp.find_module(m_path, ['frontend'])
            stat = os.stat(file.name)[8]
            if m_path in self.module_desc:
                if stat != self.module_desc[m_path]:
                    print "Reloading module", m_path
                    new_module = imp.load_module(m_path, file, pathname, description)
                    new_module.object_namespace = self.namespace
                    self.url_mapping[i] = (m[0], new_module, m_path, m[3])
            self.module_desc[m_path] = stat

    def __call__(self, environ, start_response):
        path = environ['PATH_INFO']
        found_handler = False
        for regex, module, module_path, func_name in self.url_mapping:
            if regex.match(path):
                try:
                    if self.debug:
                        self.renew_modules()
                    handler = getattr(module, func_name)
                    response = handler(request=environ)

                    # Default headers when not specified.
                    if response.response_code == '200 OK':
                        if not response.has_header('Content-Type'):
                            response['Content-Type'] = 'text/plain'
                        if not response.has_header('Content-Size'):
                            response['Content-Size'] = len(response.body)

                    found_handler = True

                except Exception, e:
                    error_info = Pyro.util.getPyroTraceback(e)
                    self.logger.error("".join(error_info))
                    if self.debug:
                        msg = ''.join(error_info)
                    else:
                        msg = ''

                    response = ResponseError(msg=msg, server_info=self.server_info)
                    found_handler = True

                break

        if not found_handler:
            response = ResponseNotFound(url=path, server_info=self.server_info)

        # Do response
        start_response(str(response.response_code), response.get_headers())
        if isinstance(response.body, unicode):
            response.body = response.body.encode('utf8')
        return [response.body]

