#-*- coding: utf-8 -*-
import lucene
import re

class SentenceTokenizer(lucene.PythonTokenizer):
    """
    Tokenizes sentences with ' ' and '\n' characters.

    Example:
    >>> import lucene
    >>> str = u"asdf\\nqwer asdfm \\ud55c\\uae00qwetsxb34\\n\\nfdfasdf as dfs  sgsg"
    >>> st = SentenceTokenizer(lucene.StringReader(str))
    >>> for a in st:
    ...     print a.toString().encode('utf-8')
    ...
    (asdf,0,4)
    (qwer,5,9)
    (asdfm,10,15)
    (한글qwetsxb34,16,27)
    (fdfasdf,29,36)
    (as,37,39)
    (dfs,40,43)
    (sgsg,45,49)
    >>> str[16:27]
    u'\ud55c\uae00qwetsxb34'
    >>> str[45:49]
    u'sgsg'
    >>> str = "쌀밥//###//차조밥/시금치#약오르지@스팍스$는말이지^^^웃고있어&&이지만||낫!그런것이지*뷁뷁.....#.#\\x10\\x13\\x13\\x10".decode('utf8')
    >>> st = SentenceTokenizer(lucene.StringReader(str))
    >>> for a in st:
    ...     print a.toString().encode('utf-8')
    ...
    (쌀밥,0,2)
    (//###//,2,9)
    (차조밥,9,12)
    (/,12,13)
    (시금치,13,16)
    (#,16,17)
    (약오르지,17,21)
    (@,21,22)
    (스팍스,22,25)
    ($,25,26)
    (는말이지,26,30)
    (^^^,30,33)
    (웃고있어,33,37)
    (&&,37,39)
    (이지만,39,42)
    (||,42,44)
    (낫,44,45)
    (!,45,46)
    (그런것이지,46,51)
    (*,51,52)
    (뷁뷁,52,54)
    (.....#.#\x10\x13\x13\x10,54,66)
    >>> str[46:51]
    u'\\uadf8\\ub7f0\\uac83\\uc774\\uc9c0'
    """
    def __init__(self, reader):
        lucene.PythonTokenizer.__init__(self, reader)
        #CharTokenizer.__init__(self, reader)
        self.input = reader
        self.str = lucene.PythonReader(reader).read((1024**2)*4)
        #self.token_regex = re.compile(ur'[  \n]')
        self.token_regex = re.compile(ur'[  \n]+|\W+|\w+', re.DOTALL| re.UNICODE)
        self.token_index = 0
        self.offset = 0
        #print self.str
        self.tokens = self.token_regex.finditer(self.str)
        #print self.tokens
        self.next_state = iter(self)

    def next(self):
        while True:
            token_string = self.tokens.next().group(0)
            length = len(token_string)
            if token_string.strip():
                break
            self.offset += length
        ret = lucene.Token(token_string, self.offset, self.offset + length)
        self.offset += length
        return ret


if __name__ == '__main__':
    import doctest
    doctest.testmod()
