# -*- coding: utf-8 -*-
import logging
import time
import traceback
import copy
import sys
import datetime
import urlparse

import Pyro.core
import Pyro.util
import lucene

from ksearch.search.analysis import koreananalyzer
from ksearch import util

class IndexerService(object):
    def __init__(self, index_dir, database_getter=None):
        self.logger = logging.getLogger("search")
        self.kanal = koreananalyzer.KoreanAnalyzer()
        self.index_dir = index_dir
        self.fs_dir = lucene.FSDirectory.getDirectory(self.index_dir)

        self.database_getter = database_getter
        self.indexing_time_limit = 10

        self._init_writer()

    def __del__(self):
        """Closes the index to clean up the instance."""
        self._close()

    def _close(self):
        self.logger.info("IndexerService: IndexerService is closing...")
        self.closed = True
        try:
            try:
                self.writer.close()
            except Exception, e:
                self.logger.error(("Error closing IndexWriter. Details below:\n%s" %
                                  traceback.format_exc()))
        finally:
            if lucene.IndexReader.isLocked(self.fs_dir):
                lucene.IndexReader.unlock(self.fs_dir)

    def _init_writer(self):
        self.closed = False

        try:
            self.writer = lucene.IndexWriter(self.fs_dir, True, self.kanal)
        except lucene.JavaError, e:
            self.logger.error("IndexWriter initialization failed: %s", e)
            sys.exit(1)
        self.writer.setUseCompoundFile(False)

    def flush(self):
        self.writer.flush()

    def optimize(self):
        self.writer.optimize()

    def doc_count(self):
        return self.writer.docCount()

    def _get_host(self, uri):
        return urlparse.urlsplit(uri)[1].rsplit(":", 1)[0]

    def _index(self, source, uri, title, content, boost, filetype, date):
        doc = lucene.Document()
        doc.setBoost(boost)
        id_ = "%s:%s" % (source, uri)
        doc.add(lucene.Field("id", id_,
                      lucene.Field.Store.YES, lucene.Field.Index.UN_TOKENIZED))
        doc.add(lucene.Field("source", source,
                      lucene.Field.Store.YES, lucene.Field.Index.UN_TOKENIZED))
        doc.add(lucene.Field("uri", uri,
                      lucene.Field.Store.YES, lucene.Field.Index.UN_TOKENIZED))
        doc.add(lucene.Field("filetype", filetype,
                      lucene.Field.Store.YES, lucene.Field.Index.UN_TOKENIZED))
        date_str = date.strftime("%Y%m%d%H%M")
        doc.add(lucene.Field("date", date_str,
                      lucene.Field.Store.YES, lucene.Field.Index.UN_TOKENIZED))
        host = self._get_host(uri)
        doc.add(lucene.Field("site", host,
                      lucene.Field.Store.YES, lucene.Field.Index.UN_TOKENIZED))
        doc.add(lucene.Field("title", title,
                      lucene.Field.Store.YES, lucene.Field.Index.TOKENIZED, lucene.Field.TermVector.WITH_POSITIONS_OFFSETS))
        doc.add(lucene.Field("body", content,
                      lucene.Field.Store.YES, lucene.Field.Index.TOKENIZED, lucene.Field.TermVector.WITH_POSITIONS_OFFSETS))
        if isinstance(content, str) or isinstance(content, unicode):
            doc.add(lucene.Field("content", uri + " " + title + " " + content,
                          lucene.Field.Store.YES, lucene.Field.Index.TOKENIZED))
        else:
            doc.add(lucene.Field("content", content,
                          lucene.Field.Store.YES, lucene.Field.Index.TOKENIZED))

        term = lucene.Term('id', id_)
        self.writer.updateDocument(term, doc)

    def index(self, source, uri, title, content, boost=1.0, filetype='html', date=None):
        """Indexes the document to the lucene index."""
        self.logger.info("Indexing: [%s] %s", source, uri)
        start = time.time()
        try:
            uri = util.smart_unicode(uri)
            title = util.smart_unicode(title)
            if date == None:
                date = datetime.datetime.now()
            assert type(date) == datetime.datetime

            self._index(source, uri, title, content, boost, filetype, date)

            database = self.database_getter()
            database.put("statistics", "total_indexed_page", self.writer.docCount())
            database.put("statistics", "total_indexed_page_in_memory",
                    self.writer.numRamDocs())

            duration = time.time() - start
            database.put("statistics", "index_time", duration)
            if duration > self.indexing_time_limit:
                self.logger.warn("Document %s took more than 10 seconds to index!", uri)

        except Pyro.errors.ConnectionClosedError:
            self.logger.error("Error indexing document %s\nDetails below:\n%s" % \
                              (uri, traceback.format_exc()))
            self.logger.error("Quitting...")
            sys.exit(1)
        except Exception, e:
            self.logger.error("Error indexing document %s\nDetails below:\n%s" % \
                              (uri, traceback.format_exc()))
